
#define	 F_CPU	16000000UL
#include <avr/io.h>
#include <util/delay.h>
#include <stdbool.h>
#include <inttypes.h>
#include "ili9341.h"
#include "ili9341gfx.h"
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>

#define SIZE    4
#define WON     1
#define LOST    2
#define CONTINUE 3
#define CONTINUE_FULL 4

extern uint16_t vsetx,vsety,vactualx,vactualy,isetx,isety,iactualx,iactualy;
static FILE mydata = FDEV_SETUP_STREAM(ili9341_putchar_printf, NULL, _FDEV_SETUP_WRITE);
int start = 0;
int won = 0;
static char values[14][5] = {"", "   2", "   4", "   8", 
                            "  16", "  32", "  64", " 128",
                            " 256", " 512", "1024", "2048", 
							"4096", "8192"};
int grid[SIZE][SIZE];

void timer1_init(void)
{
    TCCR1A &= ~((1 << COM1A0) | (1 << COM1A1) |     // OC0A dezactivat
                (1 << COM1B0) | (1 << COM1B1) |     // OC0B dezactivat
                (1 << WGM10) | (1 << WGM11));       
    TCCR1B &= ~((1 << WGM12) | (1 << WGM13) | (1 << CS11) | (1 << CS12));  // mod normal
    TCCR1B |=   (1 << CS10);                        // fara prescalar
}

void casuta_noua()
{
    uint8_t new_value, i, j;
    
    uint8_t r = rand() % 256;
    if (r > 230)
        new_value = 2;
    else
        new_value = 1;
        
    do
    {
        r = rand() % 16;
        i = r / SIZE;
        j = r % SIZE;
    }
    while (grid[i][j] != 0);
    
    grid[i][j] = new_value;
}

void initGRID(void)
{
	srand((unsigned)TCNT1);
    memset(grid, 0, SIZE * SIZE * sizeof(int));
    casuta_noua();
	casuta_noua();
}

							
static volatile uint16_t  culori[12] = {BLACK,YELLOW,OLIVE,PINK,ORANGE,RED,MAGENTA,PURPLE,CYAN,OLIVE,GREEN,BLUE,MAGENTA,YELLOW};
void drawPatratica(int i, int j, int value) {
	if(value != 0) {
		ili9341_fillrect(5+j*80,5+i*60,70,50,culori[value]);
		ili9341_setcursor(20+j*80, 25+i*60);
		ili9341_settextcolour(BLACK, culori[value]);
		ili9341_settextsize(2);
		printf("%s",values[value]);
	} else {
		ili9341_fillrect(5+j*80,5+i*60,70,50,culori[value]);
	}
}

void drawIntreg() {
	int i,j;
	for(i=0;i<SIZE;i++)
		for(j=0;j<SIZE;j++)
			drawPatratica(i,j,grid[i][j]);
}

void init(void) {
	stdout = & mydata;
	ili9341_init();//initial driver setup to drive ili9341
	ili9341_setRotation(1);//rotate screen
	ili9341_clear(BLACK);//fill screen with black colour
	timer1_init();
}


void InitADC()
{
	ADMUX = (1 << REFS0);                         // For Aref=AVcc;
	ADCSRA = (1 << ADEN) | (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0); //Rrescalar div factor =128
}

uint16_t adc_read(uint8_t ch)
{
	// select the corresponding channel 0~7
	// ANDing with '7' will always keep the value
	// of 'ch' between 0 and 7
	ch &= 0b00000111;  // AND operation with 7
	ADMUX = (ADMUX & 0xF8) | ch;     // clears the bottom 3 bits before ORing

									 // start single conversion
									 // write '1' to ADSC
	ADCSRA |= (1 << ADSC);

	// wait for conversion to complete
	// ADSC becomes '0' again
	// till then, run loop continuously
	while (ADCSRA & (1 << ADSC));

	return (ADC);
}

void move_left(uint8_t i)
{
    uint8_t c = 0;
    uint8_t j = 0;
    uint8_t u = 0;
    
    
    for (j = 0; j < SIZE; ++j)
    {
        // deplasare pana la cel mai la margine loc liber
        c = j;
        if (grid[i][j] == 0)
        {
            while (grid[i][++c] == 0 && c < SIZE);
            if (c >= SIZE)     break;
            
            grid[i][j] = grid[i][c];
            grid[i][c] = 0;
            
        }
        
        // unim ?
        if (j > 0 && u == 0 && grid[i][j - 1] == grid[i][j])
        {
            u = 1;
            ++grid[i][j - 1];
            grid[i][j] = 0;
            --j;
        }
        else
            u = 0;
    }
}

void move_right(uint8_t i)
{
    int8_t c = 0;
    uint8_t j = 0;
    uint8_t u = 0;
	
    int count =0;
	for(j=0; j < SIZE; j++)
		if(grid[i][j] == 0)
			count++;
    if(count == 0) {
		if(grid[i][0] != grid[i][1] && grid[i][1] != grid[i][2] && grid[i][2] != grid[i][3])
			return;
	}
    for (j = SIZE-1; j >= 0; --j)
    {
        // deplasare pana la cel mai la margine loc liber
        c = j;
        if (grid[i][j] == 0)
        {
            while (c >= 0 && grid[i][c] == 0) {
				c--;
			}
            if (c <= -1)     break;
			
			grid[i][j] = grid[i][c];
            grid[i][c] = 0;
            
        }
        
        // unim ?
        if (j < SIZE-1 && u == 0 && grid[i][j + 1] == grid[i][j])
        {
            u = 1;
            ++grid[i][j+1];
            grid[i][j] = 0;
            j++;
        }
        else {
            u = 0;
		}
    }
	
}

void move_up(uint8_t i)
{
    uint8_t c = 0;
    uint8_t j = 0;
    uint8_t u = 0;
    
    
    for (j = 0; j < SIZE; ++j)
    {
        // deplasare pana la cel mai la margine loc liber
        c = j;
        if (grid[j][i] == 0)
        {
            while (grid[++c][i] == 0 && c < SIZE);
            if (c >= SIZE)     break;
            
            grid[j][i] = grid[c][i];
            grid[c][i] = 0;
            
        }
        
        // unim ?
        if (j > 0 && u == 0 && grid[j - 1][i] == grid[j][i])
        {
            u = 1;
            ++grid[j - 1][i];
            grid[j][i] = 0;
            --j;
        }
        else
            u = 0;
    }
}

void move_down(uint8_t i)
{
    int8_t c = 0;
    uint8_t j = 0;
    uint8_t u = 0;
    
    
    for (j = SIZE-1; j >= 0; --j)
    {
        // deplasare pana la cel mai la margine loc liber
        c = j;
        if (grid[j][i] == 0)
        {
            while (c >= 0 && grid[c][i] == 0) {
				c--;
			}
            if (c <= -1)     break;
            
            grid[j][i] = grid[c][i];
            grid[c][i] = 0;
            
        }
        
        // unim ?
        if (j < SIZE-1 && u == 0 && grid[j + 1][i] == grid[j][i])
        {
            u = 1;
            ++grid[j+1][i];
            grid[j][i] = 0;
            ++j;
        }
        else
            u = 0;
    }
}
int evaluate(void) {
	int count = 0;
	int i,j;
	for (i = 0; i < SIZE; i++) {
		for (j = 0; j < SIZE; ++j) {
			if(grid[i][j] == 11 && won == 0) {
				won = 1;
				return WON;
			}else if(grid[i][j] == 0)
				count++;
		}
	}
	if(count!=0)
		return CONTINUE;
	else {
		for(i = 0; i < SIZE; i++) {
			if(grid[i][0] == grid[i][1] || grid[i][1] == grid[i][2] || grid[i][2] == grid[i][3])
				return CONTINUE_FULL;
			if(grid[0][i] == grid[1][i] || grid[1][i] == grid[2][i] || grid[2][i] == grid[3][i])
				return CONTINUE_FULL;
		}
		return LOST;
	}
	
}
initSw(void) {
	DDRD &= ~((1 << PD0) | (1 << PD6));
    PORTD |=  (1 << PD0) | (1 << PD6);
}

drawStart(void) {
	
	ili9341_setcursor(100, 50);
	ili9341_settextcolour(PINK, BLACK);
	ili9341_settextsize(3);
	printf("Press");
	
	ili9341_setcursor(100, 100);
	ili9341_settextcolour(PINK, BLACK);
	ili9341_settextsize(3);
	printf("to play");
	
}

drawLost(void) {
	ili9341_clear(BLACK);//fill screen with black colour
	ili9341_setcursor(20, 50);
	ili9341_settextcolour(RED, BLACK);
	ili9341_settextsize(3);
	printf("You lost :(");
	
	ili9341_setcursor(80, 100);
	ili9341_settextcolour(RED, BLACK);
	ili9341_settextsize(3);
	printf("press to");
	
	ili9341_setcursor(80, 100);
	ili9341_settextcolour(RED, BLACK);
	ili9341_settextsize(3);
	printf("restart");
	start = 0;
	
}

drawWON(void) {
	ili9341_clear(BLACK);//fill screen with black colour
	ili9341_setcursor(80, 50);
	ili9341_settextcolour(GREEN, BLACK);
	ili9341_settextsize(3);
	printf("You WON :)");
	
	ili9341_setcursor(80, 100);
	ili9341_settextcolour(GREEN, BLACK);
	ili9341_settextsize(3);
	printf("Press to ");
	
	ili9341_setcursor(80, 150);
	ili9341_settextcolour(GREEN, BLACK);
	ili9341_settextsize(3);
	printf("continue");
	start = 2;
	
}
void BUZZ(float duration,float period)
{
	int i, cycles;
	float half_period;
	cycles = duration / period;
	half_period = period / 2;
	
	for(i=0; i < cycles; i++) 
	{
		_delay_ms(half_period);
		PORTD |= (1 << PD4);
		_delay_ms(half_period);
		PORTD &= ~(1 << PD4);
	}
	return;
}
initBuzzer(void)
{
	DDRD |= (1 << PD4);
}

void decide(void) {
	int eval = evaluate();
	if(eval == CONTINUE || eval == CONTINUE_FULL) {
		if(eval == CONTINUE)
			casuta_noua();
		drawIntreg();
	} else {
		start = -1;
		if(eval == WON) {
			drawWON();
		} else {
			drawLost();
		}
	}
}
int main(void)
{
	
	int16_t x1, y1;

	InitADC();
	initSw();
	init();
	drawStart();
	initBuzzer();
	
	while (1) {
		if(start == 0 || start == 2) {
			if((PIND & (1 << PD6)) == 0 ) {
				ili9341_clear(BLACK);
				BUZZ(100,2);
				if(start == 0) //start=2 pt atunci cand se continua jocul dupa castig
					initGRID();
				drawIntreg();
				start = 1;
			}
		} else if(start == 1){

			x1 = adc_read(1);  //Read ADC Channel 0
			x1 = x1 - 512;
			y1 = adc_read(0);  //Read ADC Channel 1
			y1 = y1 - 512;

			if(y1 > 250 & x1 < 50){
				//move right
				BUZZ(100,2);
				move_right(0);
				move_right(1);
				move_right(2);
				move_right(3);
				decide();
			}else if(y1 < -250 & x1 < 50){
				BUZZ(100,2);
				move_left(0);
				move_left(1);
				move_left(2);
				move_left(3);
				decide();
			}else if(x1 > 250 & y1 < 50){
				//move down
				BUZZ(100,2);
				move_down(0);
				move_down(1);
				move_down(2);
				move_down(3);
				decide();
				
			}else if(x1 < -250 & y1 < 50){
				//move up
				BUZZ(100,2);
				move_up(0);
				move_up(1);
				move_up(2);
				move_up(3);
				decide();
				
			}
		}
	} 

		


	return 0;
}